@@ -348,12 +348,13 @@ class MidiFile:
         track_lengths = []
     
         for track in self.tracks:
-            seconds_per_tick = self._compute_tick_time(500000)
+            seconds_per_tick = self._compute_seconds_per_tick(500000)
             length = 0.0
             for message in track:
                 length += (message.time * seconds_per_tick)
                 if message.type == 'set_tempo':
-                    seconds_per_tick = self._compute_tick_time(message.tempo)
+                    seconds_per_tick = self._compute_seconds_per_tick(
+                        message.tempo)
             track_lengths.append(length)
 
         return max(track_lengths)
@@ -381,7 +382,7 @@ class MidiFile:
             # Convert message time from absolute time
             # in ticks to relative time in seconds.
             now = message.time * seconds_per_tick
-            message.time = now - time_of_last_message
+            message.time = max(0.0, now - time_of_last_message)
 
             yield message       
 
